import { Component, OnInit, ChangeDetectorRef, Renderer2, ElementRef, ViewChild, inject, QueryList, ViewChildren } from '@angular/core';
import {MatOption, provideNativeDateAdapter} from "@angular/material/core";
import {MatIconModule} from '@angular/material/icon';
import {FormBuilder, FormControl, FormGroup, FormsModule, ReactiveFormsModule, Validators} from '@angular/forms';
import {CdkTextareaAutosize} from "@angular/cdk/text-field";
import {MatAutocomplete, MatAutocompleteSelectedEvent, MatAutocompleteTrigger} from "@angular/material/autocomplete";
import {MatCard, MatCardContent, MatCardSubtitle , MatCardFooter, MatCardHeader, MatCardTitle , MatCardModule} from "@angular/material/card";
import {MatButton, MatFabButton, MatIconButton} from "@angular/material/button";
import {MatFormField, MatHint, MatLabel, MatSuffix,MatFormFieldModule} from "@angular/material/form-field";
import {MatIcon} from "@angular/material/icon";
import {MatInput} from "@angular/material/input";
import {MatTooltip} from "@angular/material/tooltip";
import {DatePipe, NgForOf, NgIf, NgStyle} from "@angular/common";
import {
  MatCell,
  MatCellDef,
  MatColumnDef,
  MatHeaderCell, MatHeaderCellDef,
  MatHeaderRow,
  MatHeaderRowDef,
  MatRow, MatRowDef, MatTable
} from "@angular/material/table";
import {MatDatepicker, MatDatepickerInput, MatDatepickerToggle} from "@angular/material/datepicker";
import {MatSelect} from "@angular/material/select";
import {SoloNumerosDirective} from "../../service/directivas_service/soloNumeros/solo-numeros.directive";
import {MatSnackBar, MatSnackBarHorizontalPosition, MatSnackBarVerticalPosition} from "@angular/material/snack-bar";
import {ConformidadHelper} from "../conformidad/helper/conformidad.helper";
import {MatDialog} from "@angular/material/dialog";
import { ConfirmarEliminacionComponent  } from '../conformidad/modalEliminarPasaje/eliminarPasaje.componente';


interface interArticulo{
  index: number;
  codigoArticulo : string;
  nombre: string;
  unidad: string;
  cantidad: number;
  estado : number; // 1 = BUENO ESTADO , 2 = OBSERVACIÓN , 3 = MAL ESTADO
  isNew: boolean;
  isEdit : boolean;
  acciones: boolean;
}

interface interEstados {
  valor: string;
  nombre: string;
}

interface interUnidad {
  codigo: string;
  nombre: string;
}

@Component({
  selector: 'ordensalida-component',
  templateUrl: './ordensalida.componente.html',
  styleUrls: ['./ordensalida.componente.css'],
  providers: [provideNativeDateAdapter()],
  standalone: true,
  imports: [
    CdkTextareaAutosize,
    FormsModule,
    MatAutocomplete,
    MatAutocompleteTrigger,
    MatCard,
    MatCardContent,
    MatCardSubtitle,
    MatFabButton,
    MatFormField,
    MatIcon,
    MatInput,
    MatLabel,
    MatOption,
    MatTooltip,
    NgForOf,
    ReactiveFormsModule,
    MatButton,
    MatCell,
    MatCellDef,
    MatColumnDef,
    MatDatepicker,
    MatDatepickerInput,
    MatDatepickerToggle,
    MatHeaderCell,
    MatHeaderRow,
    MatHeaderRowDef,
    MatHint,
    MatIconButton,
    MatRow,
    MatRowDef,
    MatSelect,
    MatSuffix,
    MatTable,
    NgIf,
    SoloNumerosDirective,
    NgStyle,
    MatHeaderCellDef,
    MatIconModule,
    MatFormFieldModule,
    MatCardModule,
    MatCardHeader,
    MatCardTitle,
    MatCardFooter,
  ]
})

export class ordenSalidaComponent implements OnInit {

  private barraAlerta = inject(MatSnackBar);
  alertaPosicionHorizontal: MatSnackBarHorizontalPosition = 'center';
  alertaPosicionVertical: MatSnackBarVerticalPosition = 'bottom';
  alertaDuracion: number = 3;


  //FORM GROUP
  public datos_materialSobrante: FormGroup = new FormGroup<any>({
    descMaterial : new FormControl<String | null>(null,[Validators.maxLength(100)]),
  });
  //FORM GROUP

  // INICIALIZACION DATOS
  columnasArticulos: string[] = ['nombre', 'unidad', 'cantidad', 'estado', 'accion'];
  articulosList: any[] = [];
  listUnidades : interUnidad[] = [];
  activarNuevoArticulo: boolean = true;
  indexArticulo: number = 0;
  public dataSource: interArticulo[] = [];
  listEstadoArticulo: interEstados[] = [{ valor: '1', nombre: 'BUENO'}, { valor: '2', nombre: 'OBSERVACIÓN' },{ valor: '3', nombre: 'MAL ESTADO' } ]
  // INICIALIZACION DATOS

  // OBTENER DATOS DEL FORM GROUP
  formGroupMS(accion: number , formControlName : string , formControl : FormControl = new FormControl()): any {

    if(accion === 1){
      return this.datos_materialSobrante.addControl(formControlName, formControl);
    }else if(accion === 2){
      //Obtener un formControl del formGroup y validar si existe caso contrario retornar null
      return this.datos_materialSobrante.get(formControlName);
    }else if(accion === 3){
      return this.datos_materialSobrante.removeControl(formControlName);
    }
  }

  constructor(
    private conformidadHelper: ConformidadHelper,
    private renderer: Renderer2,
    private detectorChange : ChangeDetectorRef,
    public dialog: MatDialog,
    private datePipe: DatePipe,
  ) {
    this.articulosList = [];
  }

  async ngOnInit(): Promise<void> {
    this.dataSource = [];

    const jsonParametrosArti = {
      accion : 5
    };
    const unidadList = await this.conformidadHelper.listadoGeneralHorizon(jsonParametrosArti) ;
    this.listUnidades = unidadList.map((item: any) => ({codigo: item.CODIGO , nombre: item.NOMBRE}));

  }

  onAgregarArticulo(event : any){
    const NuevaFila : interArticulo = {
      index: this.indexArticulo++,
      codigoArticulo: '',
      nombre: '',
      unidad: '',
      cantidad: 0,
      estado: 1,
      isNew: true,
      isEdit: false,
      acciones: true
    };

    this.formGroupMS(1,'articulo'+NuevaFila.index, new FormControl<String | null>(null,[Validators.maxLength(12)]));
    this.formGroupMS(1,'codigoArticulo'+NuevaFila.index, new FormControl<String | null>(null));
    this.formGroupMS(1,'unidad'+NuevaFila.index, new FormControl<String | null>(null));
    this.formGroupMS(1,'cantidad'+NuevaFila.index, new FormControl<String | null>(null,[Validators.maxLength(2)]));
    this.formGroupMS(1,'estado'+NuevaFila.index, new FormControl<String | null>(null ,[Validators.maxLength(1)]));

    this.dataSource.forEach((item) => {
      if (item.index !== NuevaFila.index) {
        item.isEdit = false;
        item.isNew = false;
        item.acciones = false;
      }
    });

    this.dataSource = [...this.dataSource,NuevaFila];

    this.activarNuevoArticulo = false;
    this.articulosList = [];
  }

  onEventoConfirmarArticulo($event: MouseEvent, elemento : any) {

    const articulo = this.formGroupMS(2,'articulo'+elemento.index)?.value;
    const codigoArticulo = this.formGroupMS(2,'codigoArticulo'+elemento.index)?.value;
    const unidad = this.formGroupMS(2,'unidad'+elemento.index)?.value;
    const cantidad = this.formGroupMS(2,'cantidad'+elemento.index)?.value;
    const estado = this.formGroupMS(2,'estado'+elemento.index)?.value;
    //VALIDAR LOS VALORES OBTENIDOS ANTES DE REALIZAR UN REGISTRO

    if(this.validarRegistroArticulo({articulo,cantidad,estado})){
      // Agregar los datos recogidos de la fila para agregarlos en dataSource y actualizar la tabla
      const index = this.dataSource.findIndex((item) => item.index === elemento.index);
      this.dataSource[index] = {
        index: elemento.index,
        codigoArticulo : codigoArticulo,
        nombre: articulo,
        unidad: unidad,
        cantidad: cantidad,
        estado: estado,
        isNew: false,
        isEdit : false,
        acciones: true
      };


      //console.log(this.dataSource);
      //console.log(lugarOrigen , lugarDestino, pasaje, fechaPasaje)

      this.dataSource.forEach((item) => { item.acciones = true; }); // VUELVE A ACTIVAR TODAS LAS ACCION

      this.dataSource = [...this.dataSource];
      this.activarNuevoArticulo = true;

      this.barraAlerta.open( 'REGISTRO INGRESADO' ,  '' , {
        horizontalPosition: this.alertaPosicionHorizontal,
        verticalPosition: this.alertaPosicionVertical,
        duration: this.alertaDuracion * 1000
      });

    }else{

      this.barraAlerta.open( 'VALIDAR DATOS INGRESADOS' ,  '' , {
        horizontalPosition: this.alertaPosicionHorizontal,
        verticalPosition: this.alertaPosicionVertical,
        duration: this.alertaDuracion * 1000
      });

    }

  }

  validarRegistroArticulo(valores: any): boolean {

    if (valores.articulo && valores.cantidad && valores.estado) {
      return true;
    } else {
      return false;
    }
  }

  onEventoCancelarArticulo(event: any, elemento : any) {
    const index = this.dataSource.findIndex((item) => item.index === elemento.index);//Descubriendo el index

    this.dataSource.forEach((item) => { item.acciones = true; }); // Que todas las filas vuelvan a tener las acciones

    if(this.dataSource[index].isEdit){

      this.dataSource[index].isEdit = false;
      this.dataSource[index].isNew = false;
      this.dataSource = [...this.dataSource];

    }else{

      this.dataSource = this.dataSource.filter((item) => item.index !== elemento.index);
      this.formGroupMS(3,'articulo'+elemento.index);
      this.formGroupMS(3,'cantidad'+elemento.index);
      this.formGroupMS(3,'estado'+elemento.index);

    }

    this.activarNuevoArticulo = true
  }

  onEventoEditarArticulo(event: any, elemento : any) {


    this.dataSource.forEach((item) => {
      if (item.index === elemento.index) {
        item.isEdit = true;
        item.isNew = true;
        item.acciones = true;
      } else {
        item.isEdit = false;
        item.isNew = false;
        item.acciones = false;
      }
    });

    this.dataSource = [...this.dataSource]; // Volviendo a ingresar los datos para que la tabla se actualice y la fila cambie con las condiciones del HTML
    this.activarNuevoArticulo = false;
  }

  onEventoEliminarArticulo(event: any, elemento : any) {
    //Al presionar el boton aparecerá un Dialog confirmando si desea eliminar el registro, al aceptar la fila del registro se eliminará.
    const dialogRef = this.dialog.open(ConfirmarEliminacionComponent);

    dialogRef.afterClosed().subscribe(result => {
      if (result) {
        const index = elemento.index;
        if (index >= 0) {
          this.dataSource = this.dataSource.filter((item) => item.index !== elemento.index);
          this.formGroupMS(3,'articulo'+elemento.index);
          this.formGroupMS(3,'cantidad'+elemento.index);
          this.formGroupMS(3,'estado'+elemento.index);

        }
      }
    });
  }

  async onBusquedaArticulo(event: any , index : number) { // OMITIR LA BUSQUEDA POR AHORA YA QUE NO CONOCEN LOS ARTICULOS

    const busquedaArt = event.target.value;

    const jsonParametrosArti = {
      accion : 4,
      nombreValor : busquedaArt,
    };

    const articulos_list = await this.conformidadHelper.listadoGeneralHorizon(jsonParametrosArti) ;
    const mapeoResultado = articulos_list.map((item: any) => ({
      codigo: item.codigoProducto,
      nombre : item.nombreProducto,
      unidad: item.unidad
    }));

    this.articulosList = mapeoResultado;
  }

  onSeleccionArticulo(event: MatAutocompleteSelectedEvent,index : number) { // OMITIR LA BUSQUEDA POR AHORA YA QUE NO CONOCEN LOS ARTICULOS

    //AL SELECCIONAR , SE OPTIENE EL VALUE
    const selectedOption = event.option.value;

    //SE BUSCA EL VALUE DENTRO DEL LISTADO DE ARTICULOS
    const articulo = this.articulosList.find((item) => item.codigo === selectedOption);

    //console.log(articulo);
    //SE OPTIENE EL NOMBRE PARA PONERLO EN EL INPUT
    this.formGroupMS(2,'articulo'+index)?.setValue(articulo.nombre);
    this.formGroupMS(2,'codigoArticulo'+index)?.setValue(articulo.codigo);
    this.formGroupMS(2,'unidad'+index)?.setValue(articulo.unidad);
   // this.codigoUnidad = articulo.unidad;

    //SE VUELVE A LIMPIAR EL LISTADO DE ARTICULOS PARA DEJAR PASO A OTRA FUTURA BUSQUEDA.
    this.articulosList = [];

  }

  formatoEstado(estado: any) {
    return this.listEstadoArticulo.find((item) => item.valor === estado)?.nombre;
  }
}
